@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2004-2008  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2009  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,6 @@
 void port_release_all(void);
 
 int port_register(DBusConnection *conn, const char *path, bdaddr_t *src,
-		  bdaddr_t *dst, const char *name, const char *uuid,
-		  uint8_t channel);
+		  bdaddr_t *dst, const char *name, uint8_t channel);
 
-int port_unregister(const char *path, const char *uuid);
+int port_unregister(const char *path);
